from ctypes import *
TEMUSupport = CDLL('libTEMUSupport.so')


class CpuInfo(Structure):
  _fields_ = [ 
    ('ArchName', c_char_p),
    ('ModelName', c_char_p),
    ('VASize', c_uint),
    ('PASize', c_uint),
    ('VATypeSize', c_uint),
    ('PATypeSize', c_uint),
    ('GPRCount', c_uint),
    ('FPRCount', c_uint),
    ('InstructionEndianess', c_uint),
    ('DataEndianess', c_uint),
    ('NumInstructionSets', c_uint)  ]

class CpuIface(Structure):
  _fields_ = [ 
    ('reset', CFUNCTYPE(None, c_void_p, c_int)),
    ('run', CFUNCTYPE(c_uint, c_void_p, c_uint64)),
    ('runUntil', CFUNCTYPE(c_uint, c_void_p, c_uint64)),
    ('step', CFUNCTYPE(c_uint, c_void_p, c_uint64)),
    ('stepUntil', CFUNCTYPE(c_uint, c_void_p, c_uint64, c_uint64)),
    ('raiseTrap', CFUNCTYPE(None, c_void_p, c_int)),
    ('enterIdleMode', CFUNCTYPE(None, c_void_p)),
    ('exitEmuCore', CFUNCTYPE(None, c_void_p, c_uint)),
    ('getFreq', CFUNCTYPE(c_uint64, c_void_p)),
    ('getCycles', CFUNCTYPE(c_int64, c_void_p)),
    ('getSteps', CFUNCTYPE(c_int64, c_void_p)),
    ('getState', CFUNCTYPE(c_uint, c_void_p)),
    ('setPc', CFUNCTYPE(None, c_void_p, c_uint64)),
    ('getPc', CFUNCTYPE(c_uint64, c_void_p)),
    ('setGpr', CFUNCTYPE(None, c_void_p, c_int, c_uint64)),
    ('getGpr', CFUNCTYPE(c_uint64, c_void_p, c_uint)),
    ('setFpr32', CFUNCTYPE(None, c_void_p, c_uint, c_uint32)),
    ('getFpr32', CFUNCTYPE(c_uint32, c_void_p, c_uint)),
    ('setFpr64', CFUNCTYPE(None, c_void_p, c_uint, c_uint64)),
    ('getFpr64', CFUNCTYPE(c_uint64, c_void_p, c_uint)),
    ('setSpr', CFUNCTYPE(None, c_void_p, c_uint, c_uint64)),
    ('getSpr', CFUNCTYPE(c_uint64, c_void_p, c_uint)),
    ('getRegId', CFUNCTYPE(c_int, c_void_p, c_char_p)),
    ('getRegName', CFUNCTYPE(c_char_p, c_void_p, c_int)),
    ('assemble', CFUNCTYPE(c_uint32, c_void_p, c_char_p)),
    ('disassemble', CFUNCTYPE(c_char_p, c_void_p, c_uint32)),
    ('enableTraps', CFUNCTYPE(None, c_void_p)),
    ('disableTraps', CFUNCTYPE(None, c_void_p)),
    ('invalidateAtc', CFUNCTYPE(None, c_void_p, c_uint64, c_uint64, c_uint32)),
    ('translateAddress', CFUNCTYPE(c_uint64, c_void_p, c_uint64, POINTER(c_uint32))),
    ('getPowerState', CFUNCTYPE(c_uint, c_void_p)),
    ('setPowerState', CFUNCTYPE(None, c_void_p, c_uint)),
    ('enableTrapEvents', CFUNCTYPE(None, c_void_p)),
    ('disableTrapEvents', CFUNCTYPE(None, c_void_p)),
    ('enableErrorModeEvents', CFUNCTYPE(None, c_void_p)),
    ('disableErrorModeEvents', CFUNCTYPE(None, c_void_p)),
    ('getMachine', CFUNCTYPE(c_void_p, c_void_p)),
    ('raiseTrapNoJmp', CFUNCTYPE(None, c_void_p, c_int)),
    ('getTrapName', CFUNCTYPE(c_char_p, c_void_p, c_int)),
    ('getCPUInfo', CFUNCTYPE(POINTER(CpuInfo), c_void_p)),
    ('wakeUp', CFUNCTYPE(c_int, c_void_p)),
    ('forceEarlyExit', CFUNCTYPE(None, c_void_p))  ]
